/****************************************************************************
**	File:	DBSNAPIN.C
**
**	Desc:	This snapin provides the dialogs and procedures needed to create
**				and modify the following classes of objects--
**
**			DBServer - a class that is used for a Database Server
**				to authenticate to the tree.
**			Database - a class that is used as an object to represent
**				a database in a Database Server.
**
**		DISCLAIMER
**
**	Novell, Inc. makes no representations or warranties with respect to
**	any NetWare software, and specifically disclaims any express or
**	implied warranties of merchantability, title, or fitness for a
**	particular purpose.
**
**	Distribution of any NetWare software is forbidden without the
**	express written consent of Novell, Inc.  Further, Novell reserves
**	the right to discontinue distribution of any NetWare software.
**
**	Novell is not responsible for lost profits or revenue, loss of use
**	of the software, loss of data, costs of re-creating lost data, the
**	cost of any substitute equipment or program, or claims by any party
**	other than you.  Novell strongly recommends a backup be made before
**	any software is installed.   Technical support for this software
**	may be provided at the discretion of Novell.
**
**	Programmers:
**
**		Ini	Who								Firm
**		--- ----------------- ------------------------------------------------
**		CRG	Calvin Gaisford		Novell Developer Support.
**
**	History:
**
**		When		 Who 	What
**		-------- --- 	---------------------------------------------------------
**		1-04-95  CRG 	Modification of this code for distibution. Much of this
**                  code was taken from the snapin example that comes from
**									the Novell SDK.  In fact, I just copied and pasted a lot
**									of the code that is the same for all snapin modules.
*/



/***************************************************************************
**  Libraries Linked in for .exe
**
**  NWCALLS.LIB
**  NWLOCALE.LIB
**  NWNET.LIB
**	SNAPIN.LIB
**
*/

/****************************************************************************
**	Include headers
*/
	/*-----------------------------------------------------------------------
	**	WINDOWS
	*/
	#include <windows.h>
	#include <toolhelp.h>
	#include <string.h>
	#include <stdio.h>

	/*-----------------------------------------------------------------------
	**	NetWare
	*/
	#include <nwnet.h>
	#include <nwsnapin.h>

	/*-----------------------------------------------------------------------
	**	Program
	*/
	#include "dbsnapin.h"


/***************************************************************************
**	Function Prototypes
*/

#ifdef __cplusplus
extern "C"
{
#endif

N_EXTERN_LIBRARY( NWRCODE )
SnapinDBServerProc(pnstr8 name, nuint16 msg, nparam p1, nparam p2);

N_EXTERN_LIBRARY( NWRCODE )
SnapinDBProc(pnstr8 name, nuint16 msg, nparam p1, nparam p2);

N_EXTERN_LIBRARY( nuint32 )
FADMINFlatBrowserLaunchProc( nuint32 userParam, nuint16 uMsg,
												 nparam param1, nparam param2 );
N_EXTERN_LIBRARY( nuint32 )
FOPERFlatBrowserLaunchProc( nuint32 userParam, nuint16 uMsg,
												 nparam param1, nparam param2 );
N_EXTERN_LIBRARY( nuint32 )
FNWFlatBrowserLaunchProc( nuint32 userParam, nuint16 uMsg,
												 nparam param1, nparam param2 );
N_EXTERN_LIBRARY( nuint32 )
DSFlatBrowserLaunchProc( nuint32 userParam, nuint16 uMsg,
												 nparam param1, nparam param2 );
N_EXTERN_LIBRARY( nuint32 )
DBSFlatBrowserLaunchProc( nuint32 userParam, nuint16 uMsg,
												 nparam param1, nparam param2 );

BOOL _export FAR PASCAL AddDBSObjectDialogProc(HWND hwnd, UINT message,
																						WPARAM wParam, LPARAM lParam);
BOOL _export FAR PASCAL AddDBObjectDialogProc(HWND hwnd, UINT message,
																						WPARAM wParam, LPARAM lParam);
BOOL _export FAR PASCAL DBServerIdentificationDlg(HWND hwnd, UINT message,
																						 WPARAM wParam, LPARAM lParam);
BOOL _export FAR PASCAL DatabaseIdentificationDlg(HWND hwnd, UINT message,
																						 WPARAM wParam, LPARAM lParam);
BOOL _export FAR PASCAL DBServerDatabaseDlg(HWND hwnd, UINT message,
																				WPARAM wParam, LPARAM lParam);
BOOL _export FAR PASCAL DatabaseUserDlg(HWND hwnd, UINT message,
																				WPARAM wParam, LPARAM lParam);
BOOL AddDBServerObject(char *strName);
BOOL AddDatabaseObject(char *strName, char *DBSName);
int ReadDBSObjectData(void);
int ReadDBObjectData(void);
int ModifyObjectData(char *name);
int ModifyDBObjectData(char *name);


#ifdef __cplusplus
}
#endif

/***************************************************************************
**  Type Definitions And MACROS
*/
typedef struct tagDBServerObjData
{
	HWND hDialogDatabase;
	char name[256];
	char bhnwsnameDefined;
	char hnwsname[MAX_DN_CHARS];
	char hnwsnameChange[MAX_DN_CHARS];
	int  bDescrDefined;
	char descr[1024];
	char descrChange[1024];
	int  bDatabaseDefined;
	int  nDatabaseCount;
	int  nDeltaCount;
	char databaseList[MAX_DBUSER][256];
	char databaseDeltas[MAX_DBUSER][256];
} TDBServerObject;

typedef struct tagDatabaseObjData
{
	HWND hDialogUser;
	char name[256];
	char hdbname[MAX_DN_CHARS];
	char admingroup[MAX_DN_CHARS];
	char admingroupChange[MAX_DN_CHARS];
	char opergroup[MAX_DN_CHARS];
	char opergroupChange[MAX_DN_CHARS];
	int  bOperDefined;
	int  bAdminDefined;
	int  bUserDefined;
	int  nUserCount;
	int  nDeltaCount;
	char userList[MAX_DBUSER][256];
	char userDeltas[MAX_DBUSER][256];
} TDatabaseObject;



#define NUM_DBSERVER_PAGES   2
#define NUM_DATABASE_PAGES	 2

/***************************************************************************
**  Globals Used in code
*/
NWAPageStruct dbServerPages[NUM_DBSERVER_PAGES];
NWAPageStruct databasePages[NUM_DATABASE_PAGES];
static int createAnotherFlag=0;
static int showDetailsFlag=0;
static char nameBuf[256];
static char DBSnameBuf[256];
static TDBServerObject dbServerObjData;
static TDatabaseObject databaseObjData;
static bChangeDBSId = 0;
static bChangeDBSHost = 0;
static bChangeDBSDatabase = 0;
static bChangeDBAdmin = 0;
static bChangeDBOper = 0;
static bChangeDBUser = 0;
HINSTANCE hDLL;
HINSTANCE hCtrlDLL;

/*-------------------------------------------------------------------------*/
/*    DLL Entry Point                                                      */
/*-------------------------------------------------------------------------*/
int FAR PASCAL LibMain (HINSTANCE hInstance, WORD wDataSeg,
												WORD cbHeapSize, LPSTR lpCmdLine)
{
	 hDLL = hInstance;
	 if(cbHeapSize != 0)
			UnlockData (0);
	 return 1;
};

/*-------------------------------------------------------------------------*/
/* Function : InitSnapin(void)                                             */
/* Description :                                                           */
/*    Every Snapin DLL must provide this function. In this function,       */
/*    snapin menu items (under Tools) are registered. Also, object procs   */
/*    are registered.                                                      */
/*                                                                         */
/*-------------------------------------------------------------------------*/
int _export FAR PASCAL InitSnapin()
{

	 /*----------------------------------------------------------*/
	 /* Registering DBServer Object Class Proc                   */
	 /*----------------------------------------------------------*/
	 NWARegisterObjectProc(NWA_DS_OBJECT_TYPE,
												 "DBServer",
												 "Novell, Inc. (C) All rights reserved.",
													hDLL,
													(NWASnapinObjectProc) SnapinDBServerProc,
													NWA_SNAPIN_VERSION);

	 NWARegisterObjectProc(NWA_DS_OBJECT_TYPE,
													"Database",
													"Novell, Inc. (C) All rights reserved.",
													hDLL,
													(NWASnapinObjectProc) SnapinDBProc,
													NWA_SNAPIN_VERSION);

	 return NWA_RET_SUCCESS;
}


/*-------------------------------------------------------------------------*/
/* Function : SnapinDBServerProc(void)                                     */
/* Description :                                                           */
/*   DBServer Snapin Callback Proc                                         */
/*-------------------------------------------------------------------------*/
N_GLOBAL_LIBRARY( NWRCODE ) _export
SnapinDBServerProc(pnstr8 name, nuint16 msg, nparam p1, nparam p2)
{
	 static HBITMAP hBitmap;
	 static HBITMAP hAlias;
	static HBITMAP hReadOnly;
	switch(msg)
	{
		case NWA_MSG_INITSNAPIN:
			  {
				 /*-------------------------------------------------*/
						 /*           initialize pages                      */
				 /*-------------------------------------------------*/
				 dbServerPages[0].dlgProc = DBServerIdentificationDlg;
				 dbServerPages[0].resName = "DBSERVER_IDENTIFICATION";
				 dbServerPages[0].pageTitle = "Identification";

				 dbServerPages[1].dlgProc = DBServerDatabaseDlg;
				 dbServerPages[1].resName = "DBSERVER_DATABASE";
				 dbServerPages[1].pageTitle = "Databases";

				 /*-------------------------------------------------*/
				 /*            Register bitmaps here                */
				 /*-------------------------------------------------*/
				 hBitmap   = LoadBitmap(hDLL, "DBServer");
				 hAlias    = LoadBitmap(hDLL, "DBServerAlias");
				 hReadOnly = LoadBitmap(hDLL, "DBServerRO");

				 NWAAddClassData ("DBServer", "Database Server", hBitmap, hAlias,
													 hReadOnly );

				 return NWA_RET_SUCCESS;
			 }

			case NWA_MSG_CREATEOBJECT:
					 {
						 int      rcode;
						 int      retValue;

						 retValue = DialogBox( hDLL,  "DIALOG_DBSERVER_ADD",
																 NULL, AddDBSObjectDialogProc );
						 if ( retValue )
						 {
							 lstrcat(nameBuf, ".");
							 lstrcat(nameBuf, name);
							 rcode = AddDBServerObject(nameBuf);
							 if ( rcode )
							 {
									//successfully added
									lstrcpy((char*)p1, nameBuf);
									if ( createAnotherFlag )
										 return NWA_RET_CREATEANOTHER;
									else if ( showDetailsFlag )
									{
										 showDetailsFlag=0;
										 return NWA_RET_SHOWDETAILS;
									}
							 }
							 else
								 return NWA_ERR_ERROR;
						 }
						 return NWA_RET_SUCCESS;

					 }

			case NWA_MSG_GETVALIDOPERATIONS :
					 {
						 return ( NWA_OP_DETAILS | NWA_OP_RENAME | NWA_OP_DELETE |
											NWA_OP_DSTYPE | NWA_OP_CREATE );
					 }

			case NWA_MSG_GETPAGECOUNT:
					 {
						 /*-----------------------------*/
						 /* Initialize DBServer Data    */
						 /*-----------------------------*/
						 lstrcpy(dbServerObjData.name, name);
						 dbServerObjData.descr[0]='\0';
						 dbServerObjData.nDatabaseCount=0;
						 dbServerObjData.nDeltaCount=0;
						 dbServerObjData.hnwsname[0]='\0';
						 ReadDBSObjectData();
						 return NUM_DBSERVER_PAGES;
					 }

			case NWA_MSG_REGISTERPAGE:
					 {
						 NWAPageStruct  *pageInfo =   (NWAPageStruct *)p2;
						 pageInfo->dlgProc    = dbServerPages[p1].dlgProc;
						 pageInfo->resName    = dbServerPages[p1].resName;
						 pageInfo->pageTitle  = dbServerPages[p1].pageTitle;
						 pageInfo->hDLL = hDLL;
						 pageInfo->initParam = 0;
						 return NWA_RET_SUCCESS;
					 }

			case NWA_MSG_MODIFY:
					 {
						 ModifyObjectData(name);
						 return NWA_RET_SUCCESS;
					 }

			case NWA_MSG_RENAME:
					 {
						 return NWA_RET_DODEFAULT;
					 }

			case NWA_MSG_QUERYDELETE:
					 {
						 return TRUE;
					 }

			case NWA_MSG_CLOSESNAPIN:
					 {
						 /*-------------------------------------------------*/
						 /*      unregister bitmaps and delete them         */
						 /*-------------------------------------------------*/

						 NWARemoveClassData ("DBServer", &hBitmap, &hAlias, &hReadOnly);
						 if(hBitmap)
							 DeleteObject(hBitmap);
						 if(hAlias)
							 DeleteObject(hAlias);
						 return NWA_RET_SUCCESS;
					 }

			default:
					 break;
	 }
	 return NWA_RET_SUCCESS;
}


/*-------------------------------------------------------------------------*/
/* Function : SnapinDBProc(void)                                     */
/* Description :                                                           */
/*   Database Snapin Callback Proc                                         */
/*-------------------------------------------------------------------------*/
N_GLOBAL_LIBRARY( NWRCODE ) _export
SnapinDBProc(pnstr8 name, nuint16 msg, nparam p1, nparam p2)
{
	 static HBITMAP hBitmap;
	 static HBITMAP hAlias;
	 static HBITMAP hReadOnly;
	 switch(msg)
	 {
			case NWA_MSG_INITSNAPIN:
					 {
						 /*-------------------------------------------------*/
						 /*           initialize pages                      */
						 /*-------------------------------------------------*/
						 databasePages[0].dlgProc = DatabaseIdentificationDlg;
						 databasePages[0].resName = "DATABASE_IDENTIFICATION";
						 databasePages[0].pageTitle = "Identification";

						 databasePages[1].dlgProc = DatabaseUserDlg;
						 databasePages[1].resName = "DATABASE_USER";
						 databasePages[1].pageTitle = "Users";

						 /*-------------------------------------------------*/
						 /*            Register bitmaps here                */
						 /*-------------------------------------------------*/
						 hBitmap   = LoadBitmap(hDLL, "Database");
						 hAlias    = LoadBitmap(hDLL, "DatabaseAlias");
						 hReadOnly = LoadBitmap(hDLL, "DatabaseRO");

						 NWAAddClassData ("Database", "Database", hBitmap, hAlias,
															 hReadOnly );

						 return NWA_RET_SUCCESS;
					 }

			case NWA_MSG_CREATEOBJECT:
					 {
						 int      rcode;
						 int      retValue;

						 retValue = DialogBox( hDLL,  "DIALOG_DATABASE_ADD",
																 NULL, AddDBObjectDialogProc );
						 if ( retValue )
						 {
							 lstrcat(nameBuf, ".");
							 lstrcat(nameBuf, name);
							 rcode = AddDatabaseObject(nameBuf, DBSnameBuf);
							 if ( rcode )
							 {
									//successfully added
									lstrcpy((char*)p1, nameBuf);
									if ( createAnotherFlag )
										 return NWA_RET_CREATEANOTHER;
							 }
							 else
								 return NWA_ERR_ERROR;
						 }
						 return NWA_RET_SUCCESS;

					 }

			case NWA_MSG_GETVALIDOPERATIONS :
					 {
						 return ( NWA_OP_DETAILS | NWA_OP_RENAME | NWA_OP_DELETE |
											NWA_OP_DSTYPE | NWA_OP_CREATE );
					 }

			case NWA_MSG_GETPAGECOUNT:
					 {
						 /*-----------------------------*/
						 /* Initialize DBServer Data    */
						 /*-----------------------------*/
						 lstrcpy(databaseObjData.name, name);
						 databaseObjData.nUserCount=0;
						 databaseObjData.nDeltaCount=0;
						 databaseObjData.hdbname[0]='\0';
						 databaseObjData.admingroup[0]='\0';
						 databaseObjData.opergroup[0]='\0';
						 ReadDBObjectData();
						 return NUM_DATABASE_PAGES;
					 }

			case NWA_MSG_REGISTERPAGE:
					 {
						 NWAPageStruct  *pageInfo =   (NWAPageStruct *)p2;
						 pageInfo->dlgProc    = databasePages[p1].dlgProc;
						 pageInfo->resName    = databasePages[p1].resName;
						 pageInfo->pageTitle  = databasePages[p1].pageTitle;
						 pageInfo->hDLL = hDLL;
						 pageInfo->initParam = 0;
						 return NWA_RET_SUCCESS;
					 }

			case NWA_MSG_MODIFY:
					 {
						 ModifyDBObjectData(name);
						 return NWA_RET_SUCCESS;
					 }

			case NWA_MSG_RENAME:
					 {
						 return NWA_RET_DODEFAULT;
					 }

			case NWA_MSG_QUERYDELETE:
					 {
						 return TRUE;
					 }

			case NWA_MSG_CLOSESNAPIN:
					 {
						 /*-------------------------------------------------*/
						 /*      unregister bitmaps and delete them         */
						 /*-------------------------------------------------*/

						 NWARemoveClassData ("Database", &hBitmap, &hAlias, &hReadOnly);
						 if(hBitmap)
							 DeleteObject(hBitmap);
						 if(hAlias)
							 DeleteObject(hAlias);
						 return NWA_RET_SUCCESS;
					 }

			default:
					 break;
	 }
	 return NWA_RET_SUCCESS;
}


/*-------------------------------------------------------------------------*/
/* Function : AddDBServerObject(void)                                      */
/* Description :                                                           */
/*   Adds DBServer Object in DS                                            */
/*-------------------------------------------------------------------------*/
BOOL AddDBServerObject(char *objName)
{
	 UINT                dsErr=0;
	 Buf_T              *ObjectInfo;
	 NWDSContextHandle   context;
	 uint32              flags;

	 context = NWDSCreateContext ();
	 dsErr = NWDSSetContext(context, DCK_NAME_CONTEXT, DS_ROOT_NAME);
	 dsErr = NWDSGetContext(context, DCK_FLAGS, (void *)&flags);

	 flags |= DCV_TYPELESS_NAMES;
	 flags |= DCV_XLATE_STRINGS;
	 flags |= DCV_DEREF_ALIASES;
	 flags |= DCV_DEREF_BASE_CLASS;

	 dsErr   = NWDSSetContext (context, DCK_FLAGS, (void *)&flags );

	 dsErr = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &ObjectInfo);
	 dsErr = NWDSInitBuf(context, DSV_ADD_ENTRY, ObjectInfo);
	 if(ObjectInfo)
	 {
			dsErr = NWDSPutAttrName(context, ObjectInfo, "Object Class");
			dsErr = NWDSPutAttrVal(context, ObjectInfo, SYN_CLASS_NAME, "DBServer");
			dsErr = NWDSAddObject(context, objName, NULL, 0, ObjectInfo);
			dsErr = NWDSGenerateObjectKeyPair(context, objName, "",0L);
			NWDSFreeBuf(ObjectInfo);
	 }
	 NWDSFreeContext(context);
	 if(dsErr)
			return FALSE;
	 else
			return TRUE;
}

/*-------------------------------------------------------------------------*/
/* Function : AddDatabaseObject(void)                                      */
/* Description :                                                           */
/*   Adds Database Object in DS.  Note that there is some modification     */
/*   of the Database Server as well.  Also, an ACL entry is added for the  */
/*   Database Server on the Database.  This is for functionality of the    */
/*   Database sample program.																							 */
/*-------------------------------------------------------------------------*/
BOOL AddDatabaseObject(char *objName, char *DBSName)
{
	 UINT                dsErr=0;
	 Buf_T              *ObjectInfo;
	 Buf_T							*DBSBuff;
	 NWDSContextHandle   context;
	 uint32              flags;
	 Object_ACL_T				 tmpACL;
	 char								*protAttrName="[All Attributes Rights]";

	 context = NWDSCreateContext ();
	 dsErr = NWDSSetContext(context, DCK_NAME_CONTEXT, DS_ROOT_NAME);
	 dsErr = NWDSGetContext(context, DCK_FLAGS, (void *)&flags);

	 flags |= DCV_TYPELESS_NAMES;
	 flags |= DCV_XLATE_STRINGS;
	 flags |= DCV_DEREF_ALIASES;
	 flags |= DCV_DEREF_BASE_CLASS;

	 dsErr   = NWDSSetContext (context, DCK_FLAGS, (void *)&flags );

	 tmpACL.privileges = DS_ATTR_READ;
	 tmpACL.protectedAttrName = protAttrName;
	 tmpACL.subjectName = DBSName;

	 dsErr = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &ObjectInfo);
	 dsErr = NWDSInitBuf(context, DSV_ADD_ENTRY, ObjectInfo);
	 if(ObjectInfo)
	 {
			dsErr = NWDSPutAttrName(context, ObjectInfo, "Object Class");
			dsErr = NWDSPutAttrVal(context, ObjectInfo, SYN_CLASS_NAME, "Database");
			dsErr = NWDSPutAttrName(context, ObjectInfo, "Host Server");
			dsErr = NWDSPutAttrVal(context, ObjectInfo, SYN_DIST_NAME, DBSName);
			dsErr = NWDSPutAttrName(context, ObjectInfo, "ACL");
			dsErr = NWDSPutAttrVal(context, ObjectInfo, SYN_OBJECT_ACL, &tmpACL);
			dsErr = NWDSAddObject(context, objName, NULL, 0, ObjectInfo);
			if(!dsErr)
			{
				dsErr = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &DBSBuff);
				dsErr = NWDSInitBuf(context, DSV_ADD_ENTRY, DBSBuff);
				if(DBSBuff)
				{
					dsErr = NWDSPutChange(context, DBSBuff, DS_ADD_VALUE, "User");
					dsErr = NWDSPutAttrVal(context, DBSBuff, SYN_DIST_NAME, objName);
					dsErr = NWDSModifyObject(context, DBSName, NULL, 0, DBSBuff);
					if(dsErr)
					{
						dsErr = NWDSRemoveObject(context, objName);
					}
					NWDSFreeBuf(DBSBuff);
				}
				NWDSFreeBuf(ObjectInfo);
			}
	 }
	 NWDSFreeContext(context);
	 if(dsErr)
			return FALSE;
	 else
			return TRUE;
}


/*-------------------------------------------------------------------------*/
/* Function : ModifyObjectData(void)                                       */
/* Description :                                                           */
/*  Modifys a DBServer Object Atribute Values                              */
/*-------------------------------------------------------------------------*/
int ModifyObjectData(char *name)
{
	NWDSContextHandle context;
	int               err=0;
	Buf_T             *inpBuf;
	uint32            flags;

	context = NWDSCreateContext();
	if ( (int)context == ERR_CONTEXT_CREATION )
		return ((int) context);
	err=NWDSSetContext(context, DCK_NAME_CONTEXT, DS_ROOT_NAME);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}
	err = NWDSGetContext(context, DCK_FLAGS, (void *)&flags);
	flags |= DCV_TYPELESS_NAMES;
	flags |= DCV_XLATE_STRINGS;
	flags |= DCV_DEREF_ALIASES;
	flags |= DCV_DEREF_BASE_CLASS;
	err   = NWDSSetContext (context, DCK_FLAGS, (void *)&flags );
	err=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &inpBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}
	err=NWDSInitBuf(context, DSV_MODIFY_ENTRY, inpBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		NWDSFreeBuf(inpBuf);
		return err;
	}
	/*---------------------------------------------------------------*/
	/* Modify Identification Page                                    */
	/*---------------------------------------------------------------*/
	if ( bChangeDBSId )
	{
		if ( dbServerObjData.bDescrDefined )
		{
			err = NWDSPutChange(context, inpBuf, DS_REMOVE_VALUE, "Description");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
			err = NWDSPutAttrVal(context, inpBuf, SYN_CI_STRING, dbServerObjData.descr);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
		}
		if(strlen(dbServerObjData.descrChange) > 0)
		{
			err = NWDSPutChange(context, inpBuf, DS_ADD_VALUE, "Description");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
			err = NWDSPutAttrVal(context, inpBuf, SYN_CI_STRING, dbServerObjData.descrChange);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
		}
	}
	if( bChangeDBSHost )
	{
		if ( dbServerObjData.bhnwsnameDefined )
		{
			err = NWDSPutChange(context, inpBuf, DS_REMOVE_VALUE, "Host Server");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
			err = NWDSPutAttrVal(context, inpBuf, SYN_DIST_NAME, dbServerObjData.hnwsname);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
		}
		if(strlen(dbServerObjData.hnwsnameChange) > 0)
		{
			err = NWDSPutChange(context, inpBuf, DS_ADD_VALUE, "Host Server");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
			err = NWDSPutAttrVal(context, inpBuf, SYN_DIST_NAME, dbServerObjData.hnwsnameChange);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
		}
	}

	err= NWDSModifyObject(context, name, NULL, 0, inpBuf);
	if( err < 0 )
		MessageBox(NULL, "Error Making Modification", "DBSnapin Error", MB_OK);

	NWDSFreeContext(context);
	NWDSFreeBuf(inpBuf);
	return err;
}


/*-------------------------------------------------------------------------*/
/* Function : ModifyDBObjectData(void)                                       */
/* Description :                                                           */
/*  Modifys a Database Object Atribute Values                              */
/*-------------------------------------------------------------------------*/
int ModifyDBObjectData(char *name)
{
	NWDSContextHandle context;
	int               err=0,j;
	Buf_T             *inpBuf;
	uint32            flags;
	uint32            syntaxID;

	context = NWDSCreateContext();
	if ( (int) context == ERR_CONTEXT_CREATION )
		return ((int) context);

	err=NWDSSetContext(context, DCK_NAME_CONTEXT, DS_ROOT_NAME);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	err = NWDSGetContext(context, DCK_FLAGS, (void *)&flags);

	flags |= DCV_TYPELESS_NAMES;
	flags |= DCV_XLATE_STRINGS;
	flags |= DCV_DEREF_ALIASES;
	flags |= DCV_DEREF_BASE_CLASS;

	err   = NWDSSetContext (context, DCK_FLAGS, (void *)&flags );

	err=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &inpBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	err=NWDSInitBuf(context, DSV_MODIFY_ENTRY, inpBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		NWDSFreeBuf(inpBuf);
		return err;
	}

	/*---------------------------------*/
	/* Get syntaxID of User           */
	/*---------------------------------*/
	err = NWDSGetSyntaxID(context, "User", &syntaxID);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		NWDSFreeBuf(inpBuf);
		return err;
	}

	/*-------------------------------------------------------------------*/
	/* Modify User Attribute values                                     */
	/*-------------------------------------------------------------------*/
	if ( bChangeDBUser )
	{
		if ( databaseObjData.nUserCount )
		{
			// there were some Users for the object
			// remove them first
			err = NWDSPutChange(context, inpBuf, DS_REMOVE_VALUE, "User");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}

			for(j=0; j<databaseObjData.nUserCount; j++)
			{
				err = NWDSPutAttrVal(context, inpBuf, syntaxID,
														 databaseObjData.userList[j]);
				if ( err < 0 )
				{
					NWDSFreeContext(context);
					NWDSFreeBuf(inpBuf);
					return err;
				}
			}

			// Add new list of users selected
			// if any

			if ( databaseObjData.nDeltaCount )
			{
				err = NWDSPutChange(context, inpBuf, DS_ADD_VALUE, "User");
				if ( err < 0 )
				{
					NWDSFreeContext(context);
					NWDSFreeBuf(inpBuf);
					return err;
				}

				for(j=0; j<databaseObjData.nDeltaCount; j++)
				{
					err = NWDSPutAttrVal(context, inpBuf, syntaxID,
														 databaseObjData.userDeltas[j]);
					if ( err < 0 )
					{
						NWDSFreeContext(context);
						NWDSFreeBuf(inpBuf);
						return err;
					}
				}
			}
		}
		else
		{
			// There were no users for the object to start with.
			// Add new list of users selected ( if any )

			if ( databaseObjData.nDeltaCount )
			{
				err = NWDSPutChange(context, inpBuf, DS_ADD_VALUE, "User");
				if ( err < 0 )
				{
					NWDSFreeContext(context);
					NWDSFreeBuf(inpBuf);
					return err;
				}

				for(j=0; j<databaseObjData.nDeltaCount; j++)
				{
					err = NWDSPutAttrVal(context, inpBuf, syntaxID,
														 databaseObjData.userDeltas[j]);
					if ( err < 0 )
					{
						NWDSFreeContext(context);
						NWDSFreeBuf(inpBuf);
						return err;
					}
				}
			}
		}
	}
	/*---------------------------------------------------------------*/
	/* Modify DBAdmin Attribute                                  */
	/*---------------------------------------------------------------*/
	if ( bChangeDBAdmin )
	{
		if ( databaseObjData.bAdminDefined )
		{
			err = NWDSPutChange(context, inpBuf, DS_REMOVE_VALUE, "DBAdmin");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
			err = NWDSPutAttrVal(context, inpBuf, SYN_CI_STRING, databaseObjData.admingroup);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
		}
		if(strlen(databaseObjData.admingroupChange) > 0)
		{
			err = NWDSPutChange(context, inpBuf, DS_ADD_VALUE, "DBAdmin");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
			err = NWDSPutAttrVal(context, inpBuf, SYN_CI_STRING, databaseObjData.admingroupChange);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
		}
	}
	if ( bChangeDBOper )
	{
		if ( databaseObjData.bOperDefined )
		{
			err = NWDSPutChange(context, inpBuf, DS_REMOVE_VALUE, "DBOperator");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
			err = NWDSPutAttrVal(context, inpBuf, SYN_CI_STRING, databaseObjData.opergroup);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
		}
		if(strlen(databaseObjData.opergroupChange) > 0)
		{
			err = NWDSPutChange(context, inpBuf, DS_ADD_VALUE, "DBOperator");
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
			err = NWDSPutAttrVal(context, inpBuf, SYN_CI_STRING, databaseObjData.opergroupChange);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(inpBuf);
				return err;
			}
		}
	}
	err= NWDSModifyObject(context, name, NULL, 0, inpBuf);
	if( err < 0 )
		MessageBox(NULL, "Error Making Modification", "DBSnapin Error", MB_OK);

	NWDSFreeContext(context);
	NWDSFreeBuf(inpBuf);
	return err;
}


/*-------------------------------------------------------------------------*/
/* Function : AddDBSObjectDialogProc(void)                                    */
/* Description :                                                           */
/*    Create Database Server Object Dialog Proc                            */
/*-------------------------------------------------------------------------*/
BOOL _export FAR PASCAL AddDBSObjectDialogProc(HWND hwnd, UINT message,
																				WPARAM wParam, LPARAM lParam)
{
	 int state;
	 switch(message)
	 {
			case WM_INITDIALOG:
				{
					nameBuf[0]='\0';
					SendDlgItemMessage(hwnd, IDC_ADD_ANOTHER, BM_SETCHECK,
														 createAnotherFlag, 0L);
					SendDlgItemMessage(hwnd, IDC_ADD_DETAILS, BM_SETCHECK,
														 showDetailsFlag, 0L);
					return TRUE;
				}

			case WM_COMMAND:
				{
					switch( wParam )
					{
						case IDC_CREATE :
							GetDlgItemText(hwnd, IDC_DBSERVER_NAME, nameBuf, 255);
							createAnotherFlag=(int)SendDlgItemMessage(hwnd,
																												IDC_ADD_ANOTHER,
																												BM_GETCHECK, 0, 0L);
							showDetailsFlag=(int)SendDlgItemMessage(hwnd, IDC_ADD_DETAILS,
																											BM_GETCHECK, 0, 0L);
							EndDialog(hwnd, TRUE);
							return TRUE;

						case IDC_CANCEL :
							createAnotherFlag=0;
							showDetailsFlag=0;
							EndDialog(hwnd, FALSE);
							return TRUE;

						case IDC_ADD_DETAILS :
							state =(int)  SendDlgItemMessage(hwnd, IDC_ADD_DETAILS,
																				 BM_GETCHECK, 0, 0L);
							SendDlgItemMessage(hwnd, IDC_ADD_DETAILS, BM_SETCHECK,
																 !state, 0L);
							if ( !state )
								SendDlgItemMessage(hwnd, IDC_ADD_ANOTHER, BM_SETCHECK,
																	 state, 0L);
							EnableWindow( GetDlgItem(hwnd, IDC_ADD_ANOTHER), state);
							break;

						case IDC_ADD_ANOTHER :
							state = ( int ) SendDlgItemMessage(hwnd, IDC_ADD_ANOTHER,
																				 BM_GETCHECK, 0, 0L);
							SendDlgItemMessage(hwnd, IDC_ADD_ANOTHER, BM_SETCHECK,
																 !state, 0L);
							EnableWindow( GetDlgItem(hwnd, IDC_ADD_DETAILS), state);
							break;
					}
				}

				return TRUE;

	 }
	 return FALSE ;
}


/*-------------------------------------------------------------------------*/
/* Function : AddDBObjectDialogProc(void)                                    */
/* Description :                                                           */
/*    Create Database Object Dialog Proc                                     */
/*-------------------------------------------------------------------------*/
BOOL _export FAR PASCAL AddDBObjectDialogProc(HWND hwnd, UINT message,
																				WPARAM wParam, LPARAM lParam)
{
	NWDSContextHandle context;
	int               err=0;
	char contextStr[256];
	 int state;
	 switch(message)
	 {
			case WM_INITDIALOG:
				{
					nameBuf[0]='\0';
					SendDlgItemMessage(hwnd, IDC_ADD_ANOTHER, BM_SETCHECK,
														 createAnotherFlag, 0L);
					return TRUE;
				}

			case WM_COMMAND:
				{
					switch( wParam )
					{
						case IDC_CREATE :
							GetDlgItemText(hwnd, IDC_DATABASE_NAME, nameBuf, 255);
							GetDlgItemText(hwnd, IDC_DBS_NAME, DBSnameBuf, 255);
							createAnotherFlag=(int)SendDlgItemMessage(hwnd,
																												IDC_ADD_ANOTHER,
																												BM_GETCHECK, 0, 0L);
							EndDialog(hwnd, TRUE);
							return TRUE;

						case IDC_CANCEL :
							createAnotherFlag=0;
							EndDialog(hwnd, FALSE);
							return TRUE;

						case IDC_ADD_ANOTHER :
							state = ( int ) SendDlgItemMessage(hwnd, IDC_ADD_ANOTHER,
																				 BM_GETCHECK, 0, 0L);
							SendDlgItemMessage(hwnd, IDC_ADD_ANOTHER, BM_SETCHECK,
																 !state, 0L);
							EnableWindow( GetDlgItem(hwnd, IDC_ADD_DETAILS), state);
							break;

						case IDC_FIND_DBS :
								context = NWDSCreateContext();
								if ( (int) context == ERR_CONTEXT_CREATION )
									break;

								err = NWDSGetContext(context, DCK_NAME_CONTEXT, contextStr);
								if ( err < 0 )
								{
									NWDSFreeContext(context);
									break;
								}
								databaseObjData.hDialogUser = hwnd;
								NWALaunchDSFlatBrowser(hwnd,
																			(long) &databaseObjData,
																			contextStr,
																			NULL,
																			NULL,
																			"Selected Database Server",
																			NWA_FB_SINGLE_SELECT,
																			DBSFlatBrowserLaunchProc);
							NWDSFreeContext(context);
							break;
					}
				}

				return TRUE;

	 }
	 return FALSE ;
}


/*-------------------------------------------------------------------------*/
/* Function : DBServerIdentificationDlg                                    */
/* Description :                                                           */
/*  Database Server Identification Page Dialog Proc                        */
/*-------------------------------------------------------------------------*/
BOOL _export FAR PASCAL DBServerIdentificationDlg(HWND hwnd, UINT message,
																				WPARAM wParam, LPARAM lParam)
{
	NWDSContextHandle context;
	int               err=0;
	char contextStr[256];

	 switch(message)
	 {
			case WM_INITDIALOG:
				{
					dbServerObjData.hDialogDatabase = hwnd;
					SetDlgItemText(hwnd, IDC_NAME, dbServerObjData.name);
					SetDlgItemText(hwnd, IDC_DESCR, dbServerObjData.descr);
					SetDlgItemText(hwnd, HOST_NETWARE_NAME, dbServerObjData.hnwsname);
					SendDlgItemMessage (hwnd, IDC_DESCR, EM_LIMITTEXT, 1024, 0L);
					bChangeDBSId = 0;
					bChangeDBSHost = 0;
					return TRUE;
				}

			case NWA_WM_F1HELP:
				{
					MessageBox(NULL, "Description:\n\tA text field to hold a\n\tdescription of the Server.\nHost NetWare Server:\n\tHolds the distinguished\n\tname (DN)of the NetWare\n\tServer the Database is\n\tloaded on.", "Database Server Help", MB_OK);
					return TRUE;
				}

			case WM_COMMAND:
			switch (wParam)
			{
					 case IDC_DESCR:
								if(HIWORD(lParam) == EN_CHANGE)
								{
									bChangeDBSId = 1;
									//GetDlgItemText(hwnd, IDC_DESCR, dbServerObjData.descr, 1024);
									SendMessage(hwnd, NWA_WM_SETPAGEMODIFY, (WORD)hwnd, MAKELONG(0,TRUE));
								}
								break;

					 case HOST_NETWARE_NAME:
								if(HIWORD(lParam) == EN_CHANGE)
								{
									bChangeDBSHost = 1;
									//GetDlgItemText(hwnd, HOST_NETWARE_NAME, dbServerObjData.hnwsname, MAX_DN_CHARS);
									SendMessage(hwnd, NWA_WM_SETPAGEMODIFY, (WORD)hwnd, MAKELONG(0,TRUE));
								}
								break;

					 case IDC_FIND_NETWARE:
							context = NWDSCreateContext();
							if ( (int) context == ERR_CONTEXT_CREATION )
								break;
							err = NWDSGetContext(context, DCK_NAME_CONTEXT, contextStr);
							if ( err < 0 )
							{
								NWDSFreeContext(context);
								break;
							}
							NWALaunchDSFlatBrowser(hwnd,
										 (long) &dbServerObjData,
										 contextStr,
										 NULL,
										 NULL,
										 "Selected Object",
										 NWA_FB_SINGLE_SELECT,
										 FNWFlatBrowserLaunchProc);
							break;
			}
			break;

			case NWA_WM_CANCLOSE:
			{
				// xfer data to program memory
				GetDlgItemText(hwnd, IDC_DESCR, dbServerObjData.descrChange, 1024);
				GetDlgItemText(hwnd, HOST_NETWARE_NAME, dbServerObjData.hnwsnameChange, MAX_DN_CHARS);
				return TRUE;
			}
	 }
	 return FALSE ;
}



/*-------------------------------------------------------------------------*/
/* Function : DatabaseIdentificationDlg                                    */
/* Description :                                                           */
/*  Database Identification Page Dialog Proc                               */
/*-------------------------------------------------------------------------*/
BOOL _export FAR PASCAL DatabaseIdentificationDlg(HWND hwnd, UINT message,
																						 WPARAM wParam, LPARAM lParam)
{
	NWDSContextHandle context;
	int               err=0;
	char contextStr[MAX_DN_CHARS];

	 switch(message)
	 {
			case WM_INITDIALOG:
				{
					databaseObjData.hDialogUser = hwnd;
					SetDlgItemText(hwnd, IDC_DBNAME, databaseObjData.name);
					SetDlgItemText(hwnd, IDC_HOST_DBSERVER, databaseObjData.hdbname);
					SetDlgItemText(hwnd, IDC_ADMIN_NAME, databaseObjData.admingroup);
					SetDlgItemText(hwnd, IDC_OPER_NAME, databaseObjData.opergroup);
					bChangeDBAdmin = 0;
					bChangeDBOper = 0;
					return TRUE;
				}

			case NWA_WM_F1HELP:
				{
					MessageBox(NULL, "Sample Identification Page help", "Database Help", MB_OK);
					return TRUE;
				}

			case WM_COMMAND:
			switch (wParam)
			{
				case IDC_ADMIN_NAME:
					if(HIWORD(lParam) == EN_CHANGE)
					{
						 bChangeDBAdmin = 1;
						 SendMessage(hwnd, NWA_WM_SETPAGEMODIFY, (WORD)hwnd, MAKELONG(0,TRUE));
					}
					break;
				case IDC_FIND_ADMIN:
					context = NWDSCreateContext();
					if ( (int) context == ERR_CONTEXT_CREATION )
						break;
					err = NWDSGetContext(context, DCK_NAME_CONTEXT, contextStr);
					if ( err < 0 )
					{
						NWDSFreeContext(context);
						break;
					}
					NWALaunchDSFlatBrowser(hwnd,
								 (long) &databaseObjData,
								 contextStr,
								 NULL,
								 NULL,
								 "Selected Object",
								 NWA_FB_SINGLE_SELECT,
								 FADMINFlatBrowserLaunchProc);
					break;

				case IDC_OPER_NAME:
					if(HIWORD(lParam) == EN_CHANGE)
					{
						 bChangeDBOper = 1;
						 SendMessage(hwnd, NWA_WM_SETPAGEMODIFY, (WORD)hwnd, MAKELONG(0,TRUE));
					}
					break;

				case IDC_FIND_OPER:
					context = NWDSCreateContext();
					if ( (int) context == ERR_CONTEXT_CREATION )
						break;
					err = NWDSGetContext(context, DCK_NAME_CONTEXT, contextStr);
					if ( err < 0 )
					{
						NWDSFreeContext(context);
						break;
					}
					NWALaunchDSFlatBrowser(hwnd,
								 (long) &databaseObjData,
								 contextStr,
								 NULL,
								 NULL,
								 "Selected Object",
								 NWA_FB_SINGLE_SELECT,
								 FOPERFlatBrowserLaunchProc);
					break;
			}
				break;

			case NWA_WM_CANCLOSE:
			{
				// xfer data to program memory
				GetDlgItemText(hwnd, IDC_ADMIN_NAME, databaseObjData.admingroupChange, MAX_DN_CHARS);
				GetDlgItemText(hwnd, IDC_OPER_NAME, databaseObjData.opergroupChange, MAX_DN_CHARS);
				return TRUE;
			}
	 }
	 return FALSE ;
}



/*-------------------------------------------------------------------------*/
/* Function : DatabaseUserDlg                                              */
/* Description :                                                           */
/*      Database User Page Dialog Proc                                     */
/*-------------------------------------------------------------------------*/
BOOL _export FAR PASCAL DatabaseUserDlg(HWND hwnd, UINT message,
																				WPARAM wParam, LPARAM lParam)
{
	NWDSContextHandle context;
	int               err=0;
	char contextStr[256];

	 switch(message)
	 {
			case WM_INITDIALOG:
				{
					int i;
					bChangeDBUser = 0;
					databaseObjData.hDialogUser = hwnd;
					for ( i=0; i<databaseObjData.nUserCount; i++)
						SendDlgItemMessage(hwnd, IDC_USERLIST, LB_ADDSTRING, 0,
															 (LPARAM) databaseObjData.userList[i] );
					return TRUE;
				}

			case NWA_WM_F1HELP:
				{
					MessageBox(NULL, "Sample Database User Page help", "Database Help", MB_OK);
					return TRUE;
				}

			case WM_COMMAND:
			switch (wParam)
			{
					case IDC_ADDUSER :
						{
							// launch flat browser with User and User Group
							context = NWDSCreateContext();
							if ( (int) context == ERR_CONTEXT_CREATION )
								break;
							err = NWDSGetContext(context, DCK_NAME_CONTEXT, contextStr);
							if ( err < 0 )
							{
								NWDSFreeContext(context);
								break;
							}
							NWALaunchDSFlatBrowser(hwnd,
																		 (long) &databaseObjData,
																		 contextStr,
																		 NULL,
																		 NULL,
																		 "Selected Object",
																		 NWA_FB_MULTIPLE_SELECT,
																		 DSFlatBrowserLaunchProc);
						}
						NWDSFreeContext(context);
						break;

					case IDC_DELUSER :
						{
							int selCount, i;
							int *pIndex;
							HGLOBAL hmem;

							selCount= (int)SendDlgItemMessage(hwnd, IDC_USERLIST,
																		 LB_GETSELCOUNT, 0, 0L );
							if ( selCount !=LB_ERR )
							{
								SendMessage(hwnd, NWA_WM_SETPAGEMODIFY, (WORD) hwnd,
														MAKELONG(0,TRUE));
								hmem= GlobalAlloc( GMEM_MOVEABLE | GMEM_ZEROINIT,
																	 selCount * sizeof(int));
								pIndex= (int  *) GlobalLock( hmem );
								selCount= (int)SendDlgItemMessage(hwnd, IDC_USERLIST,
																				 LB_GETSELITEMS, selCount,
																				 (long) (int FAR *) pIndex );
								for(i=selCount; i > 0 ; i--)
									SendDlgItemMessage(hwnd, IDC_USERLIST, LB_DELETESTRING,
																		 pIndex[i-1], 0L);
								GlobalUnlock(hmem);
								GlobalFree(hmem);
							}
						}
						break;
			}
				return TRUE;

			case NWA_WM_CANCLOSE:
				// xfer data from dialog memory to program memory
				// so NWA_MSG_MODIFY can save it in DS if required.
				{
					int itemCount, i;
					bChangeDBUser = 1;
					itemCount = (int) SendDlgItemMessage(hwnd, IDC_USERLIST, LB_GETCOUNT,
																				 0, 0L);
					databaseObjData.nDeltaCount=itemCount;
					for(i=0; ( i<databaseObjData.nDeltaCount ) &&
									 ( i < MAX_DBUSER ) ; i++ )
						SendDlgItemMessage(hwnd, IDC_USERLIST, LB_GETTEXT,
															 i, (LPARAM) databaseObjData.userDeltas[i]);
					return TRUE;
				}
	 }
	 return FALSE ;
}



/*-------------------------------------------------------------------------*/
/* Function : DBServerDatabaseDlg                                          */
/* Description :                                                           */
/*      Database Server's Database Page Dialog Proc                        */
/*-------------------------------------------------------------------------*/
BOOL _export FAR PASCAL DBServerDatabaseDlg(HWND hwnd, UINT message,
																				WPARAM wParam, LPARAM lParam)
{
	 switch(message)
	 {
			case WM_INITDIALOG:
				{
					int i;
					bChangeDBSDatabase=0;
					dbServerObjData.hDialogDatabase = hwnd;
					for ( i=0; i<dbServerObjData.nDatabaseCount; i++)
						SendDlgItemMessage(hwnd, IDC_DATABASELIST, LB_ADDSTRING, 0,
															 (LPARAM) dbServerObjData.databaseList[i] );
					return TRUE;
				}

			case NWA_WM_F1HELP:
				{
					MessageBox(NULL, "This is a list of the Databases that are served\nby this Database Server.  Databases are added and deleted\nfrom this list by actually creating and deleting database\nobjects.", "Database Server Help", MB_OK);
					return TRUE;
				}

			case NWA_WM_CANCLOSE:
				// xfer data from dialog memory to program memory
				// so NWA_MSG_MODIFY can save it in DS if required.
				{
					int itemCount, i;
					bChangeDBSDatabase=1;
					itemCount = (int)SendDlgItemMessage(hwnd, IDC_DATABASELIST, LB_GETCOUNT,
																				 0, 0L);
					dbServerObjData.nDeltaCount=itemCount;
					for(i=0; ( i<dbServerObjData.nDeltaCount ) &&
									 ( i < MAX_DBUSER ) ; i++ )
						SendDlgItemMessage(hwnd, IDC_DATABASELIST, LB_GETTEXT,
															 i, (LPARAM) dbServerObjData.databaseDeltas[i]);
					return TRUE;
				}
	 }
	 return FALSE ;
}


/*-------------------------------------------------------------------------*/
/* Function : FADMINFlatBrowserLaunchProc												*/
/* Description :                                                           */
/*  DSFlat Browser Callback Proc                                           */
/*-------------------------------------------------------------------------*/
N_EXTERN_LIBRARY( nuint32 ) _export
FADMINFlatBrowserLaunchProc
(
	nuint32 userParam,
	nuint16 msg,
	nparam param1,
	nparam param2
)
{
	 int fltCount;
	 NWASelObject *pObj;
	 TDatabaseObject * puData;

	 fltCount=1;

	 switch( msg )
	 {
		 case NWA_MSG_FBFILTER_COUNT :
			 {
				 *((int *)param1) = fltCount;
				 break;
			 }

		 case NWA_MSG_FBFILTER_VALUE :
			 {
				 switch( param1 )
				 {
					 case 0L:
						 strcpy( (char *) param2, "Group");
						 break;
				 }
				 break;
			 }

		 case NWA_MSG_FBOBJECT_VALUE :
			 {
				 pObj= ( NWASelObject * ) param2;
				 puData = (TDatabaseObject *) userParam;
				 SetDlgItemText(puData->hDialogUser, IDC_ADMIN_NAME, pObj->objName);
				 break;
			 }
	 }
	 return (0);
}


/*-------------------------------------------------------------------------*/
/* Function : FOPERFlatBrowserLaunchProc												*/
/* Description :                                                           */
/*  DSFlat Browser Callback Proc                                           */
/*-------------------------------------------------------------------------*/
N_EXTERN_LIBRARY( nuint32 ) _export
FOPERFlatBrowserLaunchProc
(
	nuint32 userParam,
	nuint16 msg,
	nparam param1,
	nparam param2
)
{
	 int fltCount;
	 NWASelObject *pObj;
	 TDatabaseObject * puData;

	 fltCount=1;

	 switch( msg )
	 {
		 case NWA_MSG_FBFILTER_COUNT :
			 {
				 *((int *)param1) = fltCount;
				 break;
			 }

		 case NWA_MSG_FBFILTER_VALUE :
			 {
				 switch( param1 )
				 {
					 case 0L:
						 strcpy( (char *) param2, "Group");
						 break;
				 }
				 break;
			 }

		 case NWA_MSG_FBOBJECT_VALUE :
			 {
				 pObj= ( NWASelObject * ) param2;
				 puData = (TDatabaseObject *) userParam;
				 SetDlgItemText(puData->hDialogUser, IDC_OPER_NAME, pObj->objName);
				 break;
			 }
	 }
	 return (0);
}


/*-------------------------------------------------------------------------*/
/* Function : FNWFlatBrowserLaunchProc                                      */
/* Description :                                                           */
/*  DSFlat Browser Callback Proc                                           */
/*-------------------------------------------------------------------------*/
N_EXTERN_LIBRARY( nuint32 ) _export
FNWFlatBrowserLaunchProc
(
	nuint32 userParam,
	nuint16 msg,
	nparam param1,
	nparam param2
)
{
	 int fltCount;
	 NWASelObject *pObj;
	 TDBServerObject * puData;

	 fltCount=1;

	 switch( msg )
	 {
		 case NWA_MSG_FBFILTER_COUNT :
			 {
				 *((int *)param1) = fltCount;
				 break;
			 }

		 case NWA_MSG_FBFILTER_VALUE :
			 {
				 switch( param1 )
				 {
					 case 0L:
						 strcpy( (char *) param2, "NCP Server");
						 break;
				 }
				 break;
			 }

		 case NWA_MSG_FBOBJECT_VALUE :
			 {
				 pObj= ( NWASelObject * ) param2;
				 puData = (TDBServerObject *) userParam;

				 SetDlgItemText(puData->hDialogDatabase, HOST_NETWARE_NAME, pObj->objName);
				 break;
			 }
	 }
	 return (0);
}


/*-------------------------------------------------------------------------*/
/* Function : DSFlatBrowserLaunchProc                                      */
/* Description :                                                           */
/*  DSFlat Browser Callback Proc                                           */
/*-------------------------------------------------------------------------*/
N_EXTERN_LIBRARY( nuint32 ) _export
DBSFlatBrowserLaunchProc
(
	nuint32 userParam,
	nuint16 msg,
	nparam param1,
	nparam param2
)
{
	 int fltCount;
	 NWASelObject *pObj;
	 TDBServerObject * puData;

	 fltCount=1;

	 switch( msg )
	 {
		 case NWA_MSG_FBFILTER_COUNT :
			 {
				 *((int *)param1) = fltCount;
				 break;
			 }

		 case NWA_MSG_FBFILTER_VALUE :
			 {
				 switch( param1 )
				 {
					 case 0L:
						 strcpy( (char *) param2, "DBServer");
						 break;
				 }
				 break;
			 }

		 case NWA_MSG_FBOBJECT_VALUE :
			 {
				 pObj= ( NWASelObject * ) param2;
				 puData = (TDBServerObject *) userParam;

				 SetDlgItemText(puData->hDialogDatabase, IDC_DBS_NAME, pObj->objName);
				 break;
			 }
	 }
	 return (0);
}


/*-------------------------------------------------------------------------*/
/* Function : DSFlatBrowserLaunchProc                                      */
/* Description :                                                           */
/*  DSFlat Browser Callback Proc                                           */
/*-------------------------------------------------------------------------*/
N_EXTERN_LIBRARY( nuint32 ) _export
DSFlatBrowserLaunchProc
(
	nuint32 userParam,
	nuint16 msg,
	nparam param1,
	nparam param2
)
{
	 int fltCount;
	 static long objCount;
	 NWASelObject *pObj;
	 TDatabaseObject * puData;

	 fltCount=1;

	 switch( msg )
	 {
		 case NWA_MSG_FBFILTER_COUNT :
			 {
				 *((int *)param1) = fltCount;
				 break;
			 }

		 case NWA_MSG_FBFILTER_VALUE :
			 {
				 switch( param1 )
				 {
					 case 0L:
						 strcpy( (char *) param2, "User");
						 break;
				 }
				 break;
			 }

		 case NWA_MSG_FBOBJECT_COUNT :
			 {
				 puData = (TDatabaseObject *) userParam;
				 objCount= param1;
				 SendMessage(puData->hDialogUser,
										 NWA_WM_SETPAGEMODIFY, (WORD)puData->hDialogUser,
										 MAKELONG(0,TRUE));
				 break;
			 }

		 case NWA_MSG_FBOBJECT_VALUE :
			 {
				 pObj= ( NWASelObject * ) param2;
				 puData = (TDatabaseObject *) userParam;

				 SendDlgItemMessage(puData->hDialogUser, IDC_USERLIST, LB_INSERTSTRING, -1,
													 (LPARAM)((LPSTR) pObj->objName));
				 break;
			 }
	 }
	 return (0);
}


/*-------------------------------------------------------------------------*/
/* Function : ReadDBSObjectData                                               */
/* Description :                                                           */
/*   Function for reading / modifying object data in DS                    */
/*-------------------------------------------------------------------------*/
int ReadDBSObjectData()
{
	NWDSContextHandle context;
	int               err=0, i, j;
	int32             iterHandle=-1L;
	Buf_T             *resBuf, *nameBuf;
	char              *attributes[]={"Host Server", "Description", "User"};
	uint32            totAttr, valCount, synID;
	char              attrName[MAX_DN_CHARS+1];
	uint32            flags;

	context = NWDSCreateContext();
	if ( (int) context == ERR_CONTEXT_CREATION )
		return ( (int) context );

	err=NWDSSetContext(context, DCK_NAME_CONTEXT, DS_ROOT_NAME);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	err = NWDSGetContext(context, DCK_FLAGS, (void *)&flags);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	flags |= DCV_TYPELESS_NAMES;
	flags |= DCV_XLATE_STRINGS;
	flags |= DCV_DEREF_ALIASES;
	flags |= DCV_DEREF_BASE_CLASS;

	err   = NWDSSetContext (context, DCK_FLAGS, (void *)&flags );
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	err=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &resBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	err=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &nameBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		NWDSFreeBuf(resBuf);
		return err;
	}

	/*----------------------------------------------------------------------*/
	/* Initialize input buffer and check for errors. The output buffer does */
	/* not need to be initialized. This buffer will be initialized to       */
	/* DSV_READ operations                                                  */
	/*----------------------------------------------------------------------*/

	err=NWDSInitBuf(context, DSV_READ, nameBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		NWDSFreeBuf(resBuf);
		NWDSFreeBuf(nameBuf);
		return err;
	}

	/*----------------------------------------------------------------------*/
	/* Put object attributes that are to be read into input buffer.         */
	/* Check for errors.                                                    */
	/*----------------------------------------------------------------------*/
	for(i=0; i < 3; i++)
	{
		err= NWDSPutAttrName(context, nameBuf, attributes[i]);
		if ( err < 0 )
		{
			NWDSFreeContext(context);
			NWDSFreeBuf(resBuf);
			NWDSFreeBuf(nameBuf);
			return err;
		}
	}

	/*----------------------------------------------------------------------*/
	/* Read the data.                                                       */
	/*----------------------------------------------------------------------*/

	dbServerObjData.bDescrDefined=0;
	dbServerObjData.bDatabaseDefined=0;
	dbServerObjData.bhnwsnameDefined=0;
	iterHandle=NO_MORE_ITERATIONS;
	do
	{
		err=NWDSRead(context, dbServerObjData.name, DS_ATTRIBUTE_VALUES,
								 FALSE, nameBuf, &iterHandle, resBuf);
		if ( err < 0 )
		{
			NWDSFreeContext(context);
			NWDSFreeBuf(resBuf);
			NWDSFreeBuf(nameBuf);
			return err;
		}

		/*--------------------------------------------------------------------*/
		/* pull stuff out of the output buffer, checking for errors.          */
		/* First, get total no of attributes in the buffer.                   */
		/*--------------------------------------------------------------------*/
		err=NWDSGetAttrCount(context, resBuf, &totAttr);
		if ( err < 0 )
		{
			NWDSFreeContext(context);
			NWDSFreeBuf(resBuf);
			NWDSFreeBuf(nameBuf);
			return err;
		}

		for(i=0; i< totAttr; i++)
		{
			err=NWDSGetAttrName(context, resBuf, attrName, &valCount, &synID);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(resBuf);
				NWDSFreeBuf(nameBuf);
				return err;
			}
				if(!lstrcmp(attrName,"Description"))
				{
					dbServerObjData.bDescrDefined=1;
					err=NWDSGetAttrVal(context, resBuf, synID, dbServerObjData.descr);
					if ( err < 0 )
					{
						NWDSFreeContext(context);
						NWDSFreeBuf(resBuf);
						NWDSFreeBuf(nameBuf);
						return err;
					}
				}
				else if(!lstrcmp(attrName,"Host Server"))
				{
					dbServerObjData.bhnwsnameDefined=1;
					err=NWDSGetAttrVal(context, resBuf, synID, dbServerObjData.hnwsname);
					if ( err < 0 )
					{
						NWDSFreeContext(context);
						NWDSFreeBuf(resBuf);
						NWDSFreeBuf(nameBuf);
						return err;
					}
				}
				else if(!lstrcmp(attrName,"User"))
				{
					dbServerObjData.bDatabaseDefined=1;
					dbServerObjData.nDatabaseCount = (int) valCount;
					for ( j=0; j<valCount; j++)
					{
						err=NWDSGetAttrVal(context, resBuf, synID,
															 dbServerObjData.databaseList[j]);
						if ( err < 0 )
						{
							NWDSFreeContext(context);
							NWDSFreeBuf(resBuf);
							NWDSFreeBuf(nameBuf);
							return err;
						}
					}
				}
				else
				{
					// Should not come here
					// Get the value anyway and ignore
					err=NWDSGetAttrVal(context, resBuf, synID, dbServerObjData.descrChange);
				}
		}
	} while ( iterHandle != NO_MORE_ITERATIONS );

	NWDSFreeContext(context);
	NWDSFreeBuf(resBuf);
	NWDSFreeBuf(nameBuf);
	return 0;
}



/*-------------------------------------------------------------------------*/
/* Function : ReadDBObjectData                                               */
/* Description :                                                           */
/*   Function for reading / modifying object data in DS                    */
/*-------------------------------------------------------------------------*/
int ReadDBObjectData()
{
	NWDSContextHandle context;
	int               err=0, i, j;
	int32             iterHandle=-1L;
	Buf_T             *resBuf, *nameBuf;
	char              *attributes[]={"User","Host Server", "DBAdmin", "DBOperator"};
	uint32            totAttr, valCount, synID;
	char              attrName[MAX_DN_CHARS+1];
	uint32            flags;

	context = NWDSCreateContext();
	if ( (int) context == ERR_CONTEXT_CREATION )
		return ( (int) context );

	err=NWDSSetContext(context, DCK_NAME_CONTEXT, DS_ROOT_NAME);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	err = NWDSGetContext(context, DCK_FLAGS, (void *)&flags);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	flags |= DCV_TYPELESS_NAMES;
	flags |= DCV_XLATE_STRINGS;
	flags |= DCV_DEREF_ALIASES;
	flags |= DCV_DEREF_BASE_CLASS;

	err   = NWDSSetContext (context, DCK_FLAGS, (void *)&flags );
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	err=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &resBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		return err;
	}

	err=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &nameBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		NWDSFreeBuf(resBuf);
		return err;
	}

	/*----------------------------------------------------------------------*/
	/* Initialize input buffer and check for errors. The output buffer does */
	/* not need to be initialized. This buffer will be initialized to       */
	/* DSV_READ operations                                                  */
	/*----------------------------------------------------------------------*/

	err=NWDSInitBuf(context, DSV_READ, nameBuf);
	if ( err < 0 )
	{
		NWDSFreeContext(context);
		NWDSFreeBuf(resBuf);
		NWDSFreeBuf(nameBuf);
		return err;
	}

	/*----------------------------------------------------------------------*/
	/* Put object attributes that are to be read into input buffer.         */
	/* Check for errors.                                                    */
	/*----------------------------------------------------------------------*/
	for(i=0; i < 4; i++)
	{
		err= NWDSPutAttrName(context, nameBuf, attributes[i]);
		if ( err < 0 )
		{
			NWDSFreeContext(context);
			NWDSFreeBuf(resBuf);
			NWDSFreeBuf(nameBuf);
			return err;
		}
	}

	/*----------------------------------------------------------------------*/
	/* Read the data.                                                       */
	/*----------------------------------------------------------------------*/

	databaseObjData.bOperDefined=0;
	databaseObjData.bAdminDefined=0;
	databaseObjData.bUserDefined=0;
	iterHandle=NO_MORE_ITERATIONS;
	do
	{
		err=NWDSRead(context, databaseObjData.name, DS_ATTRIBUTE_VALUES,
								 FALSE, nameBuf, &iterHandle, resBuf);
		if ( err < 0 )
		{
			NWDSFreeContext(context);
			NWDSFreeBuf(resBuf);
			NWDSFreeBuf(nameBuf);
			return err;
		}

		/*--------------------------------------------------------------------*/
		/* pull stuff out of the output buffer, checking for errors.          */
		/* First, get total no of attributes in the buffer.                   */
		/*--------------------------------------------------------------------*/
		err=NWDSGetAttrCount(context, resBuf, &totAttr);
		if ( err < 0 )
		{
			NWDSFreeContext(context);
			NWDSFreeBuf(resBuf);
			NWDSFreeBuf(nameBuf);
			return err;
		}

		for(i=0; i< totAttr; i++)
		{
			err=NWDSGetAttrName(context, resBuf, attrName, &valCount, &synID);
			if ( err < 0 )
			{
				NWDSFreeContext(context);
				NWDSFreeBuf(resBuf);
				NWDSFreeBuf(nameBuf);
				return err;
			}

			if ( lstrcmp(attrName, "User") == 0 )
			{
				databaseObjData.bUserDefined=1;
				databaseObjData.nUserCount = (int) valCount;
				for ( j=0; j<valCount; j++)
				{
					err=NWDSGetAttrVal(context, resBuf, synID,
														 databaseObjData.userList[j]);
					if ( err < 0 )
					{
						NWDSFreeContext(context);
						NWDSFreeBuf(resBuf);
						NWDSFreeBuf(nameBuf);
						return err;
					}
				}
			}
			else if ( lstrcmp(attrName, "Host Server") == 0 )
			{
				err=NWDSGetAttrVal(context, resBuf, synID, databaseObjData.hdbname);
				if ( err < 0 )
				{
					NWDSFreeContext(context);
					NWDSFreeBuf(resBuf);
					NWDSFreeBuf(nameBuf);
					return err;
				}
			}
			else if ( lstrcmp(attrName, "DBAdmin") == 0 )
			{
				databaseObjData.bAdminDefined=1;
				err=NWDSGetAttrVal(context, resBuf, synID, databaseObjData.admingroup);
				if ( err < 0 )
				{
					NWDSFreeContext(context);
					NWDSFreeBuf(resBuf);
					NWDSFreeBuf(nameBuf);
					return err;
				}
			}
			else if ( lstrcmp(attrName, "DBOperator") == 0 )
			{
				databaseObjData.bOperDefined=1;
				err=NWDSGetAttrVal(context, resBuf, synID, databaseObjData.opergroup);
				if ( err < 0 )
				{
					NWDSFreeContext(context);
					NWDSFreeBuf(resBuf);
					NWDSFreeBuf(nameBuf);
					return err;
				}
			}
			else
			{
				// Should not come here
				// Get the value anyway and ignore
				err=NWDSGetAttrVal(context, resBuf, synID, databaseObjData.opergroup);
			}
		}
	} while ( iterHandle != NO_MORE_ITERATIONS );

	NWDSFreeContext(context);
	NWDSFreeBuf(resBuf);
	NWDSFreeBuf(nameBuf);
	return 0;
}


